@@ -15,6 +15,7 @@ load('data/phenotype.test.data.Rda');
 # run pgs statistics
 pgs.test <- apply.polygenic.score(
     vcf.data = phenotype.test.data$vcf.data,
+    vcf.long.format = TRUE,
     pgs.weight.data = phenotype.test.data$pgs.weight.data,
     phenotype.data = phenotype.test.data$phenotype.data,
     missing.genotype.method = c('mean.dosage', 'normalize', 'none'),
@@ -82,6 +83,23 @@ test_that(
                 ),
             'not/a/real/directory does not exist'
             );
+        # check that user-provided pgs.columns exist
+        expect_error(
+            create.pgs.density.plot(
+                pgs.data = pgs.test,
+                pgs.columns = c('not.a.pgs.column')
+                ),
+            'pgs.columns must be a subset of the column names in pgs.data, please check your input'
+            );
+        # check that user-provided pgs.columns are numeric
+        pgs.test$non.numeric.column <- as.character(pgs.test$PGS);  # create a non-numeric column
+        expect_error(
+            create.pgs.density.plot(
+                pgs.data = pgs.test,
+                pgs.columns = c('non.numeric.column')
+                ),
+            'All columns specified in pgs.columns must be numeric'
+            );
         }
     );
 
@@ -180,7 +198,7 @@ test_that(
 
 
 test_that(
-    'create.pgs.density.plot runs correctily with user provided phenotypes', {
+    'create.pgs.density.plot runs correctly with user provided phenotypes', {
         skip.plotting.tests(skip.plots = SKIP.COMPREHENSIVE.CASES || SKIP.PLOTS);
 
         temp.dir <- tempdir();
@@ -195,7 +213,7 @@ test_that(
                 )
             );
 
-        # check handling of only continuous phenotype (not supposed to be plotted, how can I test for multipanel dimensions over a multipanel object?)
+        # check handling of only continuous phenotype (not supposed to be plotted)
         expect_no_error(
             create.pgs.density.plot(
                 pgs.data = pgs.test,
@@ -223,6 +241,51 @@ test_that(
             file.exists(file.path(temp.dir, test.filename.continuous.phenotype))
             );
 
+
+        # check handling of categorical phenotype with fewer than 2 samples in all categories
+        low.n.test.data <- pgs.test;
+        low.n.test.data$categorical.phenotype <- paste0('cat', 1:nrow(low.n.test.data));
+        expect_warning(
+            create.pgs.density.plot(
+                pgs.data = low.n.test.data,
+                phenotype.columns = c('categorical.phenotype'),
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-all-small-categories'
+                )
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.density.plot runs correctly with user provided pgs columns', {
+        skip.plotting.tests(skip.plots = SKIP.PLOTS || SKIP.COMPREHENSIVE.CASES);
+
+        temp.dir <- tempdir();
+
+        # test data with non-standard PGS column names
+        pgs.test.custom.columns <- pgs.test;
+        pgs.test.custom.columns$custom.pgs.name <- pgs.test.custom.columns$PGS.with.replaced.missing;
+        # plot pgs density
+        expect_no_error(
+            create.pgs.density.plot(
+                pgs.data = pgs.test.custom.columns,
+                pgs.columns = c('custom.pgs.name'),
+                phenotype.columns = NULL,
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-custom-pgs'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-custom-pgs',
+            file.core = 'pgs-density',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
         }
     );
 
@@ -248,6 +311,26 @@ test_that(
             'No recognized PGS columns found in pgs.data'
             );
 
+        expect_error(
+            create.pgs.with.continuous.phenotype.plot(
+                pgs.data = pgs.test,
+                phenotype.columns = 'continuous.phenotype',
+                pgs.columns = c('not.a.pgs.column'),
+                ),
+            'pgs.columns must be a subset of the column names in pgs.data, please check your input'
+            );
+
+        # check that user-provided pgs.columns are numeric
+        pgs.test$non.numeric.column <- as.character(pgs.test$PGS);  # create a non-numeric column
+        expect_error(
+            create.pgs.with.continuous.phenotype.plot(
+                pgs.data = pgs.test,
+                phenotype.columns = 'continuous.phenotype',
+                pgs.columns = c('non.numeric.column')
+                ),
+            'All columns specified in pgs.columns must be numeric'
+            );
+
         # check that phenotype.column is a character vector
         expect_error(
             create.pgs.with.continuous.phenotype.plot(
@@ -499,6 +582,39 @@ test_that(
         }
     );
 
+
+test_that(
+    'create.pgs.with.continuous.phenotype.plot works correctly with user provided pgs columns', {
+        skip.plotting.tests(skip.plots = SKIP.PLOTS || SKIP.COMPREHENSIVE.CASES);
+
+        temp.dir <- tempdir();
+
+        # test data with non-standard PGS column names
+        pgs.test.custom.columns <- pgs.test;
+        pgs.test.custom.columns$custom.pgs.name <- pgs.test.custom.columns$PGS.with.replaced.missing;
+        # plot pgs with continuous phenotype
+        expect_no_error(
+            create.pgs.with.continuous.phenotype.plot(
+                pgs.data = pgs.test.custom.columns,
+                pgs.columns = c('custom.pgs.name'),
+                phenotype.columns = 'continuous.phenotype',
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-custom-pgs'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-custom-pgs',
+            file.core = 'pgs-scatter',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        }
+    );
+
 test_that(
     'create.pgs.rank.plot correctly validates inputs', {
         # check that input data is a data frame
@@ -773,3 +889,257 @@ test_that(
 
         }
     );
+
+test_that(
+    'create.pgs.boxplot correctly validates inputs', {
+        skip.plotting.tests(skip.plots = SKIP.PLOTS);
+
+        # check that input data is a data frame
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = list()
+                ),
+            'pgs.data must be a data.frame'
+            );
+
+        # check that the required columns are present
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = data.frame(not.recognized.PGS.column = 1:10)
+                ),
+            'No recognized PGS columns found in pgs.data'
+            );
+
+        # check that phenotype.columns is a character vector
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                phenotype.columns = c(1,2,3)
+                ),
+            'phenotype.columns must be a character vector'
+            );
+        # check that phenotype.columns are present in pgs.data
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                phenotype.columns = c('missing.phenotype')
+                ),
+            'phenotype.columns must be a subset of the column names in pgs.data'
+            );
+        # check that phenotype.columns do not contain recognized PGS columns
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                phenotype.columns = c('PGS.with.replaced.missing')
+                ),
+            'phenotype.columns cannot contain recognized PGS column names'
+            );
+        # check that output.dir is a real directory
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                output.dir = 'not/a/real/directory'
+                ),
+            'not/a/real/directory does not exist'
+            );
+        # check that user-provided pgs.columns exist
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                pgs.columns = c('not.a.pgs.column')
+                ),
+            'pgs.columns must be a subset of the column names in pgs.data, please check your input'
+            );
+        # check that user-provided pgs.columns are numeric
+        pgs.test$non.numeric.column <- as.character(pgs.test$PGS);  # create a non-numeric column
+        expect_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                pgs.columns = c('non.numeric.column')
+                ),
+            'All columns specified in pgs.columns must be numeric'
+            );
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs with no error with basic inputs', {
+        skip.plotting.tests(skip.plots = SKIP.PLOTS);
+
+        temp.dir <- tempdir();
+
+        # plot pgs boxplot
+        expect_no_error(
+            create.pgs.boxplot(
+                pgs.data = subset(pgs.test, select = -c(PGS.with.replaced.missing, PGS.with.normalized.missing)),
+                phenotype.columns = NULL,
+                output.dir = temp.dir,
+                filename.prefix = 'TEST'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST',
+            file.core = 'pgs-boxplot',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        # check returned object
+        test.plot.object <- create.pgs.boxplot(
+            pgs.data = pgs.test,
+            filename.prefix = 'TEST',
+            output.dir = NULL
+            );
+        expect_equal(
+            class(test.plot.object),
+            'multipanel'
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs correctly with tidy titles enabled', {
+        skip.plotting.tests(skip.plots = SKIP.COMPREHENSIVE.CASES || SKIP.PLOTS);
+
+        temp.dir <- tempdir();
+
+        # plot pgs boxplot
+        expect_no_error(
+            create.pgs.boxplot(
+                pgs.data = subset(pgs.test, select = -c(PGS.with.replaced.missing, PGS.with.normalized.missing)),
+                phenotype.columns = NULL,
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-tidy-titles',
+                tidy.titles = TRUE
+                )
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs correctly with user provided phenotypes', {
+        skip.plotting.tests(skip.plots = SKIP.COMPREHENSIVE.CASES || SKIP.PLOTS);
+
+        temp.dir <- tempdir();
+
+        # check handling of many phenotypes
+        expect_no_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test,
+                phenotype.columns = c('continuous.phenotype', 'binary.phenotype', 'categorical.phenotype'),
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-all-phenotypes'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-all-phenotypes',
+            file.core = 'pgs-boxplot',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs correctly with large number of phenotype categories', {
+        skip.plotting.tests(skip.plots = SKIP.COMPREHENSIVE.CASES || SKIP.PLOTS);
+
+        temp.dir <- tempdir();
+
+        # extend test data data frame by repeating itself a bunch of times
+        very.categorical.pgs.test <- pgs.test[rep(1:nrow(pgs.test), 73 * 4), ];
+        # simulate a PGS column
+        very.categorical.pgs.test$PGS <- rnorm(nrow(very.categorical.pgs.test));
+        # add more categories to categorical phenotype
+        very.categorical.pgs.test$very.categorical.phenotype <- as.character(rep(1:73, each = nrow(pgs.test)));
+
+        expect_warning(
+            create.pgs.boxplot(
+                pgs.data = very.categorical.pgs.test,
+                phenotype.columns = 'very.categorical.phenotype',
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-many-categories'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-many-categories',
+            file.core = 'pgs-boxplot',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs correctly with user provided pgs columns', {
+        skip.plotting.tests(skip.plots = SKIP.COMPREHENSIVE.CASES || SKIP.PLOTS);
+
+        temp.dir <- tempdir();
+
+        # test data with non-standard PGS column names
+        pgs.test.custom.columns <- pgs.test;
+        pgs.test.custom.columns$custom.pgs.name <- pgs.test.custom.columns$PGS.with.replaced.missing;
+        # plot pgs boxplot
+        expect_no_error(
+            create.pgs.boxplot(
+                pgs.data = pgs.test.custom.columns,
+                pgs.columns = c('custom.pgs.name'),
+                phenotype.columns = NULL,
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-custom-pgs'
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-custom-pgs',
+            file.core = 'pgs-boxplot',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        }
+    );
+
+test_that(
+    'create.pgs.boxplot runs correctly with toggled stripplot option', {
+        skip.plotting.tests(skip.plots = SKIP.PLOTS || SKIP.COMPREHENSIVE.CASES);
+
+        temp.dir <- tempdir();
+
+        # plot pgs boxplot with stripplot
+        expect_no_error(
+            create.pgs.boxplot(
+                pgs.data = subset(pgs.test, select = -c(PGS.with.replaced.missing, PGS.with.normalized.missing)),
+                phenotype.columns = NULL,
+                output.dir = temp.dir,
+                filename.prefix = 'TEST-stripplot',
+                add.stripplot = FALSE
+                )
+            );
+
+        test.filename <- generate.filename(
+            project.stem = 'TEST-stripplot',
+            file.core = 'pgs-boxplot',
+            extension = 'png'
+            );
+        expect_true(
+            file.exists(file.path(temp.dir, test.filename))
+            );
+
+        }
+    );
